/*-
 * Written by Dridi Boukelmoune <dridi.boukelmoune@gmail.com>
 *
 * This file is in the public domain.
 *
 * HPACK: Static Table Definition (RFC 7540 Appendix A)
 */

/*lint -save -e525 -e539 */

HPS( 1,	":authority",			"")
HPS( 2,	":method",			"GET")
HPS( 3,	":method",			"POST")
HPS( 4,	":path",			"/")
HPS( 5,	":path",			"/index.html")
HPS( 6,	":scheme",			"http")
HPS( 7,	":scheme",			"https")
HPS( 8,	":status",			"200")
HPS( 9,	":status",			"204")
HPS(10,	":status",			"206")
HPS(11,	":status",			"304")
HPS(12,	":status",			"400")
HPS(13,	":status",			"404")
HPS(14,	":status",			"500")
HPS(15,	"accept-charset",		"")
HPS(16,	"accept-encoding",		"gzip, deflate")
HPS(17,	"accept-language",		"")
HPS(18,	"accept-ranges",		"")
HPS(19,	"accept",			"")
HPS(20,	"access-control-allow-origin",	"")
HPS(21,	"age",				"")
HPS(22,	"allow",			"")
HPS(23,	"authorization",		"")
HPS(24,	"cache-control",		"")
HPS(25,	"content-disposition",		"")
HPS(26,	"content-encoding",		"")
HPS(27,	"content-language",		"")
HPS(28,	"content-length",		"")
HPS(29,	"content-location",		"")
HPS(30,	"content-range",		"")
HPS(31,	"content-type",			"")
HPS(32,	"cookie",			"")
HPS(33,	"date",				"")
HPS(34,	"etag",				"")
HPS(35,	"expect",			"")
HPS(36,	"expires",			"")
HPS(37,	"from",				"")
HPS(38,	"host",				"")
HPS(39,	"if-match",			"")
HPS(40,	"if-modified-since",		"")
HPS(41,	"if-none-match",		"")
HPS(42,	"if-range",			"")
HPS(43,	"if-unmodified-since",		"")
HPS(44,	"last-modified",		"")
HPS(45,	"link",				"")
HPS(46,	"location",			"")
HPS(47,	"max-forwards",			"")
HPS(48,	"proxy-authenticate",		"")
HPS(49,	"proxy-authorization",		"")
HPS(50,	"range",			"")
HPS(51,	"referer",			"")
HPS(52,	"refresh",			"")
HPS(53,	"retry-after",			"")
HPS(54,	"server",			"")
HPS(55,	"set-cookie",			"")
HPS(56,	"strict-transport-security",	"")
HPS(57,	"transfer-encoding",		"")
HPS(58,	"user-agent",			"")
HPS(59,	"vary",				"")
HPS(60,	"via",				"")
HPS(61,	"www-authenticate",		"")
#undef HPS

/*lint -restore */
